package cp;


import cp.db.DB;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;

public class Round {
  private Games games;
  private int roundNr = 0;
  private Tournament tournament;
  
  public Round(Tournament tournament) {
    this.tournament = tournament;
    games = new Games(tournament);  
  }
  
  public Round(Tournament tournament, int roundNr, Games games) {
    this.tournament = tournament;
    this.roundNr = roundNr;
    this.games = games;
    // Update games and game to point to this tournament
    games.setTournament(tournament);
  }
  
  boolean isEmpty() {
    return games.size() == 0;
  }
  
  boolean hasAllResults() {
    return games.hasAllResults();
  }

  public void setGames(Games games) {
    this.games = games;
  }

  public Games getGames() {
    return games;
  }

  public void setRoundNr(int roundNr) {
    this.roundNr = roundNr;
  }

  public int getRoundNr() {
    return roundNr;
  }
  
  public void save() throws SQLException {
    // Delete the old round if it exists and save the round.
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    String sql = "DELETE from tournamentgames " +
                 "WHERE tournament = " + tournament.getIndex() + " " + 
                 "AND round = " + roundNr;
    try {
      s.execute(sql);
      games.save(roundNr);
      conn.commit();
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
  }
  
  public void delete() throws SQLException {
    // Delete the old round if it exists and save the round.
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    String sql = "DELETE from tournamentgames " +
                 "WHERE tournament = " + tournament.getIndex() + " " + 
                 "AND round = " + roundNr;
    try {
      s.execute(sql);
      conn.commit();
      games = new Games(tournament);
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
  }

  public void load(int roundNr, Players players) throws SQLException, ChessPairerException {
    this.roundNr = roundNr;
    games = new Games(tournament);
    games.load(roundNr, players);
  }
  
  public String toString() {
    return String.format("Rond %d", roundNr);
  }

  public void append(Game game) throws SQLException {
    games.add(game);
    game.save(roundNr, games.size());
  }

  int getLotNr(Player player) throws ChessPairerException {
    for (int i=0; i<games.size(); i++) {
      Game game = games.get(i);
      Player gamePlayer = game.getWhitePlayer();
      if (gamePlayer != null && gamePlayer.getIndex() == player.getIndex()) {
        return gamePlayer.getLotNr();
      }
      gamePlayer = game.getBlackPlayer();
      if (gamePlayer != null && gamePlayer.getIndex() == player.getIndex()) {
        return gamePlayer.getLotNr();
      }
    }
    throw new ChessPairerException("Spelare ej funnen i rond");
  }

  public Tournament getTournament() {
    return tournament;
  }

	/**
	 * Returns the currently paired players.
	 */
	public Players getPairedPlayers() {
		Players players = new Players(tournament.getSettings());
	  for (int i=0; i<games.size(); i++) {
			Game game = games.get(i);
			Player white = game.getWhitePlayer();
			Player black = game.getBlackPlayer();
			if (white != null) {
				players.add(white);
			}
			if (black != null) {
				players.add(black);
			}
		}
		return players;
	}

	public void exportToLiveChess(FileWriter fileWriter) throws IOException {
	  for (int i=0; i<games.size(); i++) {
	    games.get(i).exportToLiveChess(fileWriter, roundNr, i+1);
		}
	}
}
