package cp.gui;

import cp.*;
import cp.db.DB;
import cp.db.Settings;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.RowSorter;
import javax.swing.ScrollPaneConstants;
import javax.swing.SortOrder;
import javax.swing.table.TableRowSorter;

public class AvailablePlayersDlg extends JDialog implements PlayerEditor {
  private EditPlayerPanel editPlayerPanel = new EditPlayerPanel();
  private JScrollPane jScrollPane = new JScrollPane();
  private GridBagLayout gridBagLayout1 = new GridBagLayout();
  private JPanel jPanelList = new JPanel();
  private JButton jButtonDelete = new JButton();
  private JLabel jLabel5 = new JLabel();
  private Settings settings;
  private PlayerListTableModel model;
  private JTable jTable;
  private JButton jButtonClose = new JButton();
  private Players players;
  private TableRowSorter<PlayerListTableModel> sorter;


  public AvailablePlayersDlg(Settings settings, Image icon) {
    this.settings = settings;
    model = new PlayerListTableModel();
    jTable = new JTable(model);
    this.setIconImage(icon);
    try {
      jbInit();
      editPlayerPanel.init(settings, this);
      this.getRootPane().setDefaultButton(editPlayerPanel.getDefaultButton());
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void setVisible(boolean value) {
    // Load available players
    try {
      loadAvailablePlayers();
    }
    catch (SQLException e) {
      JOptionPane.showMessageDialog(this, "SQL fel:" + e.getMessage(), "Fel", JOptionPane.ERROR_MESSAGE);
    }
    editPlayerPanel.updateClubList();
    if (value) {
      update();
    }
    super.setVisible(value);
  }

  private void jbInit() throws Exception {
    this.getContentPane().setLayout(gridBagLayout1);
    this.setTitle("ndra i spelarpool");
    this.setModal(true);
    this.setResizable(true);
    jScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    jScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    jScrollPane.setPreferredSize(new Dimension(480, 12));
    jButtonDelete.setText("Ta bort");
    jButtonDelete.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButtonDelete_actionPerformed();
      }
    });
    jLabel5.setText("Spelarpool");
    jLabel5.setFont(new Font("Tahoma", 1, 11));
    jTable.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        jButtonEdit_actionPerformed();
      }
    });
    jButtonClose.setText("Stng");
    jButtonClose.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButtonClose_actionPerformed();
      }
    });
    this.getContentPane().add(editPlayerPanel, new GridBagConstraints(0, 0, 1, 2, 0.0, 0.0, GridBagConstraints.NORTH, GridBagConstraints.BOTH, new Insets(0, 5, 0, 5), 0, 0));
    jScrollPane.getViewport().add(jTable, null);
    this.getContentPane().add(jScrollPane, new GridBagConstraints(1, 0, GridBagConstraints.REMAINDER, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(15, 0, 0, 0), 0, 0));
    jPanelList.add(jButtonDelete, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 0, 0));
    this.getContentPane().add(jPanelList, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    this.getContentPane().add(jLabel5, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    this.getContentPane().add(jButtonClose, new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 8), 0, 0));
    // Setup column sorting
    sorter = new TableRowSorter<PlayerListTableModel>(model);
    jTable.setRowSorter(sorter);
    java.util.List<RowSorter.SortKey> sortKeys = new ArrayList<RowSorter.SortKey>();
    sortKeys.add(new RowSorter.SortKey(1, SortOrder.ASCENDING)); // Namn
    sortKeys.add(new RowSorter.SortKey(3, SortOrder.ASCENDING)); // Klass
    sorter.setSortKeys(sortKeys);
    pack();
  }

  private void loadAvailablePlayers() throws SQLException {
    players = new Players(settings);
    players.load();
    model.setPlayers(players);
    jTable.clearSelection();
  }

  private void jButtonClose_actionPerformed() {
    this.setVisible(false);
  }

  private void jButtonEdit_actionPerformed() {
    int row = jTable.getSelectedRow();
    if (row > -1) {
      int modelRow = jTable.convertRowIndexToModel(row);
      Player player = model.getPlayer(modelRow);
      if (player != null) {
        editPlayerPanel.setPlayer(player);
      }
    }
  }

  private void jButtonDelete_actionPerformed() {
    int[] rows = jTable.getSelectedRows();
    if (rows.length > -1) {
      if (JOptionPane.showConfirmDialog(this, "r du sker p att du vill ta bort valda spelare?", "Bekrfta radera", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.OK_OPTION) {
        Connection conn = DB.getConnection();
        try {
          for (int i=rows.length-1; i>=0; i--) {
            int modelRow = jTable.convertRowIndexToModel(rows[i]);
            Player player = model.getPlayer(modelRow);
            player.remove();
            players.remove(player);
          }
          conn.commit();
        }
        catch (SQLException f) {
          try {
            conn.rollback();
            JOptionPane.showMessageDialog(this, "SQL fel:" + f.getMessage(), "Fel", JOptionPane.ERROR_MESSAGE);
          }
          catch (SQLException g) {
            JOptionPane.showMessageDialog(this, "SQL fel:" + g.getMessage(), "Fel", JOptionPane.ERROR_MESSAGE);
          }
        }
        update();
        editPlayerPanel.setPlayer(null);
        jTable.clearSelection();
      }
    }
  }

  public void add(Player player) throws SQLException {
    player.addToAvailablePlayers();
    players.add(player);
    update();
  }

  public void update(Player player) throws SQLException {
    player.update();
    update();
    jTable.clearSelection();
  }

  public boolean showTournamentInfo() {
    return false;
  }

  void update() {
    if (model != null) {
      model.update();
      model.setColumnWidths(jTable);
    }
    jTable.doLayout();
    jTable.repaint();
  }
  
  public Tournament getTournament() {
    return null;
  }
}
