package cp.gui;

import cp.Round;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.print.PrinterException;
import java.text.MessageFormat;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ScrollPaneConstants;
import javax.swing.table.TableColumn;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class StandingsPanel extends JScrollPane {
	private StandingsTableModel standingsTableModel;
	private JTable jTable, jPrintTable;

	public StandingsPanel() {
		try {
			standingsTableModel = new StandingsTableModel();
			jTable = new JTable(standingsTableModel);
			jPrintTable = new JTable(standingsTableModel);
			jbInit();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void jbInit() throws Exception {
		this.setViewportView(jTable);
		jTable.setFillsViewportHeight(true);
		setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		this.addComponentListener(new ComponentAdapter() {
			public void componentShown(ComponentEvent e) {
				this_componentShown();
			}
		});
		jTable.setShowGrid(false);
		jTable.setShowHorizontalLines(true);
		jPrintTable.setShowGrid(false);
		jPrintTable.setShowHorizontalLines(true);
	}

	public void setRound(Round round) {
		standingsTableModel.setRound(round);
	}

	void print() throws PrinterException {
		update();
		Round round = standingsTableModel.getRound();
		if (round != null) {
			MessageFormat headerFormat = new MessageFormat(String.format("Stllning efter rond %d", round.getRoundNr()));
			MessageFormat footerFormat = new MessageFormat(String.format("%s, grupp %s\n     - Sida {0} -", round.getTournament(), round.getTournament().getGroup()));
			EditResultsPanel.optimizeTableWidth(jPrintTable);
			jPrintTable.print(JTable.PrintMode.FIT_WIDTH, headerFormat, footerFormat);
		}
	}

	public Round getRound() {
		return standingsTableModel.getRound();
	}

	public void save(Document document, Element body) {
		// Insert table
		Element table = document.createElement("table");
		body.appendChild(table);
		table.setAttribute("class", "CP_TableStandings");
		table.setAttribute("cellpadding", "0");
		table.setAttribute("cellspacing", "0");
		table.setAttribute("border", "0");
		// Create headers
		Element tr = document.createElement("tr");
		tr.setAttribute("class", "CP_TableStandingsHeader");
		table.appendChild(tr);
		for (int i = 0; i < standingsTableModel.getColumnCount(); i++) {
		  String header = standingsTableModel.getColumnName(i);
			Element th = document.createElement("th");
			tr.appendChild(th);
			th.setAttribute("class", String.format("CP_TableStandingsHeader%s", header));
			Text text = document.createTextNode(header);
			th.appendChild(text);
		}
		for (int i = 0; i < standingsTableModel.getRowCount(); i++) {
			tr = document.createElement("tr");
			table.appendChild(tr);
			tr.setAttribute("class", "CP_TableStandingsRow");
			for (int j = 0; j < standingsTableModel.getColumnCount(); j++) {
				Element th = document.createElement("th");
				tr.appendChild(th);
				if (standingsTableModel.getColumnName(j).equalsIgnoreCase("Pong")) {
					th.setAttribute("class", String.format("CP_TableStandingsCScore", j + 1));
				}
				else {
					th.setAttribute("class", String.format("CP_TableStandingsC%d", j + 1));
				}
				Object obj = standingsTableModel.getValueAt(i, j);
				if (obj != null) {
					Text text = document.createTextNode(obj.toString());
					th.appendChild(text);
				}
			}
		}
	}

	public void update() {
		if (standingsTableModel != null) {
			standingsTableModel.update();
			setColumnWidths();
		}
		jTable.doLayout();
		jTable.repaint();
	}

	private void setColumnWidths() {
		setColumnWidths(jTable);
		setColumnWidths(jPrintTable);
	}

	private void setColumnWidths(JTable table) {
		TableColumn column = null;
		for (int i = 0; i < table.getColumnModel().getColumnCount(); i++) {
			column = table.getColumnModel().getColumn(i);
      String header = (String)column.getHeaderValue();
		  if (header.equalsIgnoreCase("Plac")) {
				column.setPreferredWidth(30);
			}
		  else if (header.equalsIgnoreCase("Namn")) {
				column.setPreferredWidth(200);
			}
		  else if (header.equalsIgnoreCase("Grp")) {
				column.setPreferredWidth(40);
			}
		  else if (header.equalsIgnoreCase("Klubb")) {
				column.setPreferredWidth(100);
			}
			else {
				column.setPreferredWidth(45);
			}
		}
	}

	private void this_componentShown() {
		update();
	}
}
