package cp.pairing;

import cp.ChessPairerException;
import cp.Game;
import cp.Games;
import cp.Player;
import cp.Players;
import cp.Round;
import cp.ScoreLotnrComparator;
import cp.Tournament;
import java.util.Collections;
import java.util.Random;
import java.util.Vector;

public abstract class PairingSystem {
  Tournament tournament;
  Players players;
  
  public PairingSystem(Tournament tournament) {
    this.tournament = tournament;
    this.players = tournament.getPlayers();
  }
  
  void initialize() {
    // First sort players according to rating
    Collections.sort(players);
    Random randomizer = new Random();
    for (int i = 0; i < players.size(); i++) {
      Player player = players.get(i);
      if (tournament.getInitialPairing().equals("Slumpad") || tournament.getInitialPairing().equals("Random")) {
        player.setLotNr(randomizer.nextInt(1000000000));
      }
      else {
        player.setLotNr(i + 1);
      }
    }
  }

  public Games pair() throws ChessPairerException {
    Games games = new Games(tournament);
    Player bye = null;
    // Handle withdrawn players
    Vector<Player> withdrawn = withdrawPlayers();
    // Handle bye
    if (players.size() % 2 == 1) {
      bye = findNextByePlayer(players);
      players.remove(bye);
    }

    // Pair all games
    Games newGames = pair(players);

    // Add bye player
    if (bye != null) {
      players.add(bye);
      if (newGames != null) {
        Game byeGame = new Game(games, bye, null);
        newGames.add(byeGame);
      }
    }
    // Add withdrawn players
    players.addAll(withdrawn);
    return newGames;
  }
  
  protected abstract Games pair(Players players) throws ChessPairerException;
  
  Player findNextByePlayer(Players players) {
    Round round = null;
    if ( tournament.getRounds().size() > 0) {
      round = tournament.getRounds().lastElement();
    }
    Collections.sort(players, new ScoreLotnrComparator(round));
    for (int i = players.size() - 1; i >= 0; i--) {
      Player player = players.get(i);
      if (!player.hasMet(null)) {
        return player;
      }
    }
    return null;
  }
  
  public boolean hasMet(Player a, Player b, Games games) {
    for (int i = 0; i < games.size(); i++) {
      Game game = games.get(i);
      if (game.hasMet(a, b)) {
        return true;
      }
    }
    return false;
  }
  
  protected boolean isBarred(Player a, Player b) {
    if (tournament.isBarredPairing()) {
      if (a.getClub() != null && b.getClub() != null) {
        return (a.getClub().toString().equals(b.getClub().toString()));
      }
    }
    return false;
  }
  
  Vector<Player> withdrawPlayers() {
    Vector<Player> withdrawn = new Vector<Player>();
    for (int i = 0; i < players.size(); i++) {
      Player player = players.get(i);
      if (player.isWithdrawn(tournament.getRounds().size() + 1)) {
        players.remove(player);
        withdrawn.add(player);
        i--;
      }
    }
    return withdrawn;
  }
}
