package cp.tiebreak;

import cp.*;

import cp.Result.*;
import java.util.*;

public class TiebreakRatingPerformance implements Tiebreak {
  
  /**
   * Calculates the tiebreak score for a player up to a given round.
   */
  public double getTiebreak(Round round, Player player) {
  	// For each opponent that has the same score, add the internal result.
    Vector games = player.getGames();
    ArrayList<Player> opponents = new ArrayList<Player>();
    for (int i = 0; i < games.size(); i++) {
      Game game = (Game)games.get(i);
      if (game.getRoundNr() <= round.getRoundNr()) {
        opponents.add(game.getOpponent(player));
      }
    }
    return getLASKPerformance(opponents, player.getScore(round), round.getRoundNr());
  }
  
  public String toString() {
    return "Prestationsrating LASK";
  }

  public String getShortName() {
    return "Pres";
  }
  
  public String format(double value) {
    return Integer.toString((int) value);
  }
  
  public static double getLASKPerformance(ArrayList<Player> opponents, double resultScore, int rounds) {
    int i = 0;
    int opponentSum = 0;
    while (i<opponents.size()) {
      Player opponent = opponents.get(i);
      if (opponent != null) {
        opponentSum += opponent.getRatingN();
      }
      i++;
    }
    int average = opponentSum / opponents.size();
    double fracScore = resultScore / rounds;
    boolean minusFrac = false;
    if (fracScore <0.50) {
        minusFrac = true;
        fracScore = 1-fracScore;
    }
    fracScore = Math.abs(fracScore);
  
    int addon = 0;
    if (fracScore > 0.99)
        addon = 800;
    else if (fracScore > 0.98)
        addon = 677;
    else if (fracScore > 0.97)
        addon = 589;
    else if (fracScore > 0.96)
        addon = 538;
    else if (fracScore > 0.95)
        addon = 501;
    else if (fracScore > 0.94)
        addon = 470;
    else if (fracScore > 0.93)
        addon = 444;
    else if (fracScore > 0.92)
        addon = 422;
    else if (fracScore > 0.91)
        addon = 401;
    else if (fracScore > 0.90)
        addon = 383;
    else if (fracScore > 0.89)
        addon = 366;
    else if (fracScore > 0.88)
        addon = 351;
    else if (fracScore > 0.87)
        addon = 336;
    else if (fracScore > 0.86)
        addon = 322;
    else if (fracScore > 0.85)
        addon = 309;
    else if (fracScore > 0.84)
        addon = 296;
    else if (fracScore > 0.83)
        addon = 284;
    else if (fracScore > 0.82)
        addon = 273;
    else if (fracScore > 0.81)
        addon = 262;
    else if (fracScore > 0.80)
        addon = 251;
    else if (fracScore > 0.79)
        addon = 240;
    else if (fracScore > 0.78)
        addon = 230;
    else if (fracScore > 0.77)
        addon = 220;
    else if (fracScore > 0.76)
        addon = 211;
    else if (fracScore > 0.75)
        addon = 202;
    else if (fracScore > 0.74)
        addon = 193;
    else if (fracScore > 0.73)
        addon = 184;
    else if (fracScore > 0.72)
        addon = 175;
    else if (fracScore > 0.71)
        addon = 166;
    else if (fracScore > 0.70)
        addon = 158;
    else if (fracScore > 0.69)
        addon = 149;
    else if (fracScore > 0.68)
        addon = 141;
    else if (fracScore > 0.67)
        addon = 133;
    else if (fracScore > 0.66)
        addon = 125;
    else if (fracScore > 0.65)
        addon = 117;
    else if (fracScore > 0.64)
        addon = 110;
    else if (fracScore > 0.63)
        addon = 102;
    else if (fracScore > 0.62)
        addon = 95;
    else if (fracScore > 0.61)
        addon = 87;
    else if (fracScore > 0.60)
        addon = 80;
    else if (fracScore > 0.59)
        addon = 72;
    else if (fracScore > 0.58)
        addon = 65;
    else if (fracScore > 0.57)
        addon = 57;
    else if (fracScore > 0.56)
        addon = 50;
    else if (fracScore > 0.55)
        addon = 43;
    else if (fracScore > 0.54)
        addon = 36;
    else if (fracScore > 0.53)
        addon = 29;
    else if (fracScore > 0.52)
        addon = 21;
    else if (fracScore > 0.51)
        addon = 14;
    else if (fracScore > 0.50)
        addon = 7;
    if (minusFrac)
        addon = -1*addon;
    return average+addon;
  }
}
