package cp.update;

import cp.db.*;
import java.io.*;

import java.net.UnknownHostException;

import java.sql.SQLException;

import javax.swing.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

public class Update implements Runnable {
  private Element rootNode;
  private String currentVersion;
  private Settings settings;
  private boolean silent;

  public Update(String currentVersion, Settings settings, boolean silent) {
    this.currentVersion = currentVersion;
    this.settings = settings;
    this.silent = silent;
  }
  
  public void init(String location) throws ParserConfigurationException, SAXException, IOException {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder documentBuilder = factory.newDocumentBuilder();
    documentBuilder.setErrorHandler(new ErrorHandler() {
        public void warning(SAXParseException e) throws SAXParseException {
          throw e;
        }
        public void error(SAXParseException e) throws SAXParseException {
          throw e;
        }
        public void fatalError(SAXParseException e) throws SAXParseException {
          throw e;
        }
      });
    Document document = documentBuilder.parse(location);
    rootNode = document.getDocumentElement();
  }
  
  public boolean updateAvailable() {
    NodeList nodeList = rootNode.getElementsByTagName("version");
    if (nodeList.getLength() > 0) {
      Node node = nodeList.item(0);
      NamedNodeMap nodeMap = node.getAttributes();
      String version = nodeMap.getNamedItem("name").getNodeValue();
      return !version.equalsIgnoreCase(currentVersion);
    }
    return false;
  }
  
  private String getCurrentVersion() {
    NodeList nodeList = rootNode.getElementsByTagName("version");
    if (nodeList.getLength() > 0) {
      Node node = nodeList.item(0);
      NamedNodeMap nodeMap = node.getAttributes();
      return nodeMap.getNamedItem("name").getNodeValue();
    }
    return null;
  }
  
  private String getUpdateXMLFile() {
    NodeList nodeList = rootNode.getElementsByTagName("location");
    if (nodeList.getLength() > 0) {
      Node node = nodeList.item(0);
      NamedNodeMap nodeMap = node.getAttributes();
      return nodeMap.getNamedItem("xml").getNodeValue();
    }
    return null;
  }

  private String getUpdateWebPage() {
    NodeList nodeList = rootNode.getElementsByTagName("location");
    if (nodeList.getLength() > 0) {
      Node node = nodeList.item(0);
      NamedNodeMap nodeMap = node.getAttributes();
      return nodeMap.getNamedItem("webpage").getNodeValue();
    }
    return null;
  }

  public void run() {
    try {
      // Get local updatelocation
      String dbUpdateLocation = settings.getUpdateLocation();
      init(dbUpdateLocation);
      // Check if updatelocation has changed
      String xmlUpdateLocation = getUpdateXMLFile();
      if (xmlUpdateLocation != null && !xmlUpdateLocation.equalsIgnoreCase(dbUpdateLocation)) {
        Settings.setUpdateLocation(xmlUpdateLocation);
      }
      // Check for update
      if (updateAvailable()) {
        JOptionPane.showMessageDialog(null, String.format("%s finns nu tillgnglig p %s", getCurrentVersion(), getUpdateWebPage()) , "Uppdatering funnen!", JOptionPane.INFORMATION_MESSAGE);
      }
      else {
        if (!silent) {
          JOptionPane.showMessageDialog(null, "Du kr senaste versionen.", "Uppdatering", JOptionPane.INFORMATION_MESSAGE);
        }
      }
    }
    catch (SQLException e) {
      if (!silent) {
        JOptionPane.showMessageDialog(null, e.getMessage(), "Fel: " + e.getMessage(), JOptionPane.ERROR_MESSAGE);
      }
    }
    catch (UnknownHostException e) {
      if (!silent) {
        JOptionPane.showMessageDialog(null, "Kan inte kontakta " + e.getMessage(), "Information", JOptionPane.INFORMATION_MESSAGE);
      }
    }
    catch (IOException e) {
      if (!silent) {
        JOptionPane.showMessageDialog(null, "Kan inte lsa xml-fil:" + e.getMessage(), "Fel", JOptionPane.ERROR_MESSAGE);
      }
    }
    catch (SAXException e) {
      JOptionPane.showMessageDialog(null, "Fel i xml-fil: " + e.getMessage(), "Fel", JOptionPane.ERROR_MESSAGE);
    }
    catch (ParserConfigurationException e) {
      JOptionPane.showMessageDialog(null, "Fel i xml-fil: " + e.getMessage(), "Fel", JOptionPane.ERROR_MESSAGE);
    }
  }

}
